# -*- coding: utf-8 -*-


import re
import sys


# TODO 补全
ACCEPT_LANGUAGE_MAP = {
    "jp": "ja-JP,ja;q=0.9",
    "us": "en-US,en;q=0.9",
    "tw": "zh-TW,zh;q=0.9",
    "hk": "zh-HK,zh;q=0.9",
    "it": "it-IT,it;q=0.9",
    "de": "de-DE,de;q=0.9",
    "es": "es-ES,es;q=0.9",
    "in": "en-IN,en;q=0.9",
    "cn": "zh-CN,zh;q=0.9",
    "fr": "fr-FR,fr;q=0.9",
    "tr": "tr-TR,tr;q=0.9",
    "ru": "ru-RU,uk;q=0.9",
    "gb": "en-GB,en;q=0.9",
    "ua": "uk-UA,uk;q=0.9",
    "ca": "en-US,en;q=0.9",
    "au": "en-AU,en;q=0.9"
}


def parse_sec_ch_ua(user_agent: str) -> str:
    """
    根据 user-agent 计算 sec-ch-ua 值
    :param user-agent:
    :return
    """
    version = int(re.search(r"(\d+)\.0\.0\.0", user_agent)[1])
    greasey_chars = [
        " ",
        "(",
        ":",
        "-",
        ".",
        "/",
        ")",
        ";",
        "=",
        "?",
        "_",
    ]
    greased_versions = ["8", "99", "24"]
    orders = [
        [0, 1, 2],
        [0, 2, 1],
        [1, 0, 2],
        [1, 2, 0],
        [2, 0, 1],
        [2, 1, 0],
    ][version % 6]
    brands = [
        {
            "brand": "".join([
                "Not",
                greasey_chars[version % 11],
                "A",
                greasey_chars[(version + 1) % 11],
                "Brand",
            ]),
            "version": greased_versions[version % 3],
        },
        { "brand": "Chromium", "version": str(version) },
        { "brand": "Google Chrome", "version": str(version) },
    ]
    _brands = [None, None, None]
    _brands[orders[0]] = brands[0]
    _brands[orders[1]] = brands[1]
    _brands[orders[2]] = brands[2]
    
    return ", ".join(map(lambda _: f'"{_["brand"]}";v="{_["version"]}"', _brands))



# -*- coding: utf-8 -*-

import re
import sys


from typing import Dict, Optional, Union, Tuple, List

if sys.version_info >= (3, 8):  # pragma: no cover (py38+)
    from typing import Literal
else:  # pragma: no cover (py38+)
    from typing_extensions import Literal

import rnet
import random
import json
from urllib.parse import urlencode
from .headers import parse_sec_ch_ua, ACCEPT_LANGUAGE_MAP
from .response import Response
from .cookies import Cookie


class Session:
    
    def __init__(
        self, 
        user_agent: Union[int, Tuple[int, Literal["windows", "macos"], Literal["chrome", "edge"]]] = None,
        cookies: Optional[Dict[str, str]] = None, proxy: Optional[str] = None, ipinfo: dict = {}, timeout: int = 15,
        allow_redirects: bool = True, verify: bool = False, http_version: Optional[Literal["http1", "http2"]] = None, sync: bool = True
    ):
        self.cookies = Cookie(cookies or {})
        
        if not user_agent:
            version = random.randint(115, 137)
            user_agent = random.choice([
                f"Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/{version}.0.0.0 Safari/537.36",
                f'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/{version}.0.0.0 Safari/537.36',  
                f'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/{version}.0.0.0 Safari/537.36 Edg/{version}.0.0.0',
                f"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/{version}.0.0.0 Safari/537.36 Edg/{version}.0.0.0",
            ])

        if isinstance(user_agent, tuple):
            user_agent_version, user_agent_os, user_agent_brand = user_agent        
            if not user_agent_version:
                user_agent_version = random.randint(115, 137)
            if isinstance(user_agent_version, tuple):
                user_agent_version = random.randint(user_agent_version[0], user_agent_version[1])
            user_agent = {
                "macos": {
                    "chrome": f'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/{user_agent_version}.0.0.0 Safari/537.36',
                    "edge": f"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/{user_agent_version}.0.0.0 Safari/537.36 Edg/{user_agent_version}.0.0.0",
                },
                "windows": {
                    "chrome": f"Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/{user_agent_version}.0.0.0 Safari/537.36",
                    "edge": f'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/{user_agent_version}.0.0.0 Safari/537.36 Edg/{user_agent_version}.0.0.0',
                },
            }[user_agent_os or random.choice(["macos", "windows"])][user_agent_brand or random.choice(["chrome", "edge"])]
        
        self.user_agent = user_agent

        sec_ch_ua_ptf = '"macOS"' if 'Mac' in user_agent else '"Windows"'
        sec_ch_ua = parse_sec_ch_ua(user_agent)
        
        self.client_hints = {
            'sec-ch-ua': sec_ch_ua,
            'sec-ch-ua-platform': sec_ch_ua_ptf,
            'user-agent': user_agent, 
            'accept-language': 'zh-CN,zh;q=0.9'
        }
        
        self._chrome_version = int(re.search(r'Chrome\/(\d+)\.\d+\.\d+\.\d+', user_agent)[1])
            
        if 'Edg' in user_agent:
            impersonate_brand = "Edge"
            impersonate_versions = [122, 127, 131, 134]
        else:
            impersonate_brand = "Chrome"
            impersonate_versions = [114, 116, 117, 118, 119, 120, 123, 124, 126, 127, 128, 129, 130, 131, 132, 133, 134, 135, 136]
        
        min_version = None
        impersonate_version = None
        for v in impersonate_versions:
            cv = abs(v - self._chrome_version)
            if not min_version or cv < min_version:
                min_version = cv
                impersonate_version = v
                
        impersonate = f'{impersonate_brand}{impersonate_version}'
        
        self.proxy = proxy
        proxies = []
        if proxy:
            if not proxy.startswith("http"):
                if "://" in proxy:
                    proxy = proxy.split("://")[1]
            
            self.proxy = proxy            
            proxies.append(rnet.Proxy.all(url=proxy))

        http_version_cfg = {
            "http1_only": False, "http2_only": False
        }
        if http_version == "http1":
            http_version_cfg["http1_only"] = True
        elif http_version == "http2":
            http_version_cfg["http2_only"] = True
            
        self._session = (rnet.BlockingClient if sync else rnet.Client)(
            impersonate=rnet.ImpersonateOption(
                impersonate=getattr(rnet.Impersonate, impersonate),
                impersonate_os=rnet.ImpersonateOS.MacOS if "Mac" in user_agent else rnet.ImpersonateOS.Windows,
            ),
            referer=True,
            user_agent=user_agent,
            timeout=timeout,
            cookie_store=False,
            no_keepalive=False,
            verify=verify,
            allow_redirects=allow_redirects,
            **http_version_cfg,
            proxies=proxies
        )
        
        self.ipinfo = ipinfo or {}
        
        if self.ipinfo.get("country"):
            self.client_hints['accept-language'] = ACCEPT_LANGUAGE_MAP.get(self.ipinfo["country"], 'zh-CN,zh;q=0.9')
    
    def __getattr__(self, name):
        """
        当访问 Session 没有定义的属性或方法时，尝试从 self._session 获取。
        这样可以支持 self._session 的所有方法和属性。
        """
        # 获取 self._session 的属性或方法
        attr = getattr(self._session, name)

        # 如果是方法，返回一个包装器以确保调用时正确传递参数
        if callable(attr):
            def wrapper(*args, **kwargs):
                return attr(*args, **kwargs)
            return wrapper
        else:
            return attr

    def update_ipinfo(self):
        try:
            resp = self._session.get("https://ipinfo.io/json", headers={
                "user-agent": self.user_agent,
            }, timeout=5).json()
            self.ipinfo.update({
                "country": resp["country"].lower(),
                "ip": resp.get("ip"),
                "timezone": resp.get("timezone"),
                "geolocation": resp.get("loc")
            })
            self.client_hints['accept-language'] = ACCEPT_LANGUAGE_MAP.get(self.ipinfo["country"], 'zh-CN,zh;q=0.9')
        except:
            pass
            
    def complete_headers(self, headers: Union[Dict[str, str], List[str], str]):
        """
        补全请求头
        :param headers: 传入的请求头
        :return
        """
        processed_headers = rnet.HeaderMap({})
        
        if isinstance(headers, str):
            headers = headers.split("\n")
        
        # 补全 cookie
        if isinstance(headers, list):
            for header in headers:
                key, value = [_.strip() for _ in header.split(":", 1)]
                if key.lower() == "cookie":
                    set_cookies = value.split(";")
                    for set_cookie in set_cookies:
                        set_cookie_key, set_cookie_value = set_cookie.strip().split("=", 1)
                        self.cookies[set_cookie_key] = set_cookie_value
                    continue                        

                processed_headers.append(key, value)
            
            if len(self.cookies) > 0:
                processed_headers.append('Cookie', self.cookies.to_str())
            
            return processed_headers
        
        set_cookies = headers.get("Cookie") or headers.get("cookie")
        if set_cookies:
            for set_cookie in set_cookies.split(";"):
                set_cookie_key, set_cookie_value = set_cookie.strip().split("=", 1)
                self.cookies[set_cookie_key] = set_cookie_value
        
        if len(self.cookies) > 0:
            if 'Cookie' in headers:           
                headers['Cookie'] = self.cookies.to_str()
            else:
                headers['cookie'] = self.cookies.to_str()
        
        # 处理 priority 124 开始有 priority
        priority = headers.get('priority') or headers.get('Priority')
        
        if self._chrome_version <= 123 and priority:
            if 'Priority' in headers:
                del headers['Priority']
                    
            if 'priority' in headers:
                del headers['priority']
        
        # 补全 accept
        set_accept = headers.get('accept') or headers.get('Accept')
        if not set_accept:
            headers['Accept'] = '*/*'
        
        # 补全 accept-language
        accept_language = None
        if self.ipinfo.get('country'):
            accept_language = ACCEPT_LANGUAGE_MAP.get(self.ipinfo['country'], "zh-CN,zh;q=0.9")

        # ipnifo 格式
        elif self.ipinfo.get('language'):
            accept_language = self.ipinfo['language'] + ',' + self.ipinfo['language'].split('-')[0] + ';q=0.9'
        
        set_accept_language = headers.get('accept-language') or headers.get('Accept-Language')
        if accept_language:
            if not set_accept_language:
                headers['Accept-Language'] = accept_language
            else:
                if ';' not in set_accept_language and set_accept_language != accept_language:
                    headers['Accept-Language'] = accept_language
                    if 'accept-language' in headers:
                        del headers['accept-language']
        
        # 更新 ua 以及 client_hints
        if 'User-Agent' in headers:
            headers['User-Agent'] = self.user_agent
        else:
            headers['user-agent'] = self.user_agent
        
        if 'sec-ch-ua-platform' in headers:
            headers['sec-ch-ua-platform'] = self.client_hints['sec-ch-ua-platform']
        else:
            headers['Sec-Ch-Ua-Platform'] = self.client_hints['sec-ch-ua-platform']

        if 'sec-ch-ua' in headers:
            headers['sec-ch-ua'] = self.client_hints['sec-ch-ua']
        else:
            headers['Sec-Ch-Ua'] = self.client_hints['sec-ch-ua']
        
        for key, value in headers.items():
            processed_headers.append(key, value)
                
        return processed_headers
    
    def set_headers_order(self, headers: Union[Dict[str, str], rnet.HeaderMap]):
        """
        请求头排序
        :param headers: 用户传入的请求头
        :return
        """
        if isinstance(headers, rnet.HeaderMap):
            headers_order = [k.decode() for k, _ in headers.items()][::-1]
        else:
            headers_order = list(headers.keys())
        
        if self._chrome_version > 130:
            # cookie 在 priority 前面
            if 'priority' in headers_order and 'cookie' in headers_order:
                priority_idx = headers_order.index('priority')
                cookie_idx = headers_order.index('cookie')
                
                if cookie_idx > priority_idx:        
                    headers_order[priority_idx] = 'cookie'
                    headers_order[cookie_idx] = 'priority'
        
        self._session.update(headers_order=headers_order)

    def _process_get_arguments(self, **kwargs):
        if kwargs.get("params"):
            kwargs["query"] = []
            for k, v in kwargs["params"].items():
                kwargs['query'].append((k, v))

        headers = kwargs.get('headers') or {}
        
        # 补全请求头
        headers = self.complete_headers(headers)
        
        # 请求头排序
        self.set_headers_order(headers)

        kwargs['headers'] = headers
        
        return {**kwargs}

    def _process_post_arguments(self, **kwargs):
        headers = kwargs.get('headers') or {}
            
        if kwargs.get("params"):
            kwargs["query"] = []
            for k, v in kwargs["params"].items():
                kwargs['query'].append((k, v))
            del kwargs["params"]

        if kwargs.get("data"):
            kwargs["body"] = urlencode(kwargs["data"]).encode()
            del kwargs["data"]
            
            if isinstance(headers, dict):
                if 'Content-Type' in headers:
                    headers['Content-Type'] = 'application/x-www-form-urlencoded'
                else:
                    headers['content-type'] = 'application/x-www-form-urlencoded'
            elif isinstance(headers, list):
                if any([header.lower().startswith('content-type') for header in headers]) == 0:
                    headers.append('content-type: application/x-www-form-urlencoded')
                        
        if kwargs.get('json'):
            kwargs["body"] = json.dumps(kwargs["json"], separators=(",", ":")).encode()
            del kwargs["json"]

            if isinstance(headers, dict):
                if 'Content-Type' in headers:
                    headers['Content-Type'] = 'application/json'
                else:
                    headers['content-type'] = 'application/json'
            elif isinstance(headers, list):
                if any([header.lower().startswith('content-type') for header in headers]) == 0:
                    headers.append('content-type: application/json')
                        
        if kwargs.get("body"):
            content_length = str(len(kwargs['body']))
            
            if isinstance(headers, dict):
                if 'Content-Length' in headers:
                    headers['Content-Length'] = content_length
                else:
                    headers["content-length"] = content_length
            else:
                if any([header.lower().startswith('content-length') for header in headers]) == 0:
                    headers.insert(0, f'content-length: {content_length}')
        
        # 补全请求头
        headers = self.complete_headers(headers)
        
        # 请求头排序
        self.set_headers_order(headers)

        kwargs['headers'] = headers
        
        return {**kwargs}
    
    def get(self, url: str, *args, **kwargs) -> Response:
        resp = Response(self._session.get(url, *args, **self._process_get_arguments(**kwargs)))
        self.cookies.update(resp.cookies)
        return resp
    
    def post(self, url: str, *args, **kwargs) -> Response:
        # 发起请求
        resp = Response(self._session.post(url, *args, **self._process_post_arguments(**kwargs)))
        self.cookies.update(resp.cookies)
        return resp


class AsyncSession(Session):

    def __init__(
        self,
        user_agent: Union[int, Tuple[int, Literal["windows", "macos"], Literal["chrome", "edge"]]] = None,
        cookies: Optional[Dict[str, str]] = None, proxy: Optional[str] = None, ipinfo: dict = {}, timeout: int = 15,
        allow_redirects: bool = True, verify: bool = False, http_version: Optional[Literal["http1", "http2"]] = None
    ):
        super().__init__(
            user_agent=user_agent, cookies=cookies, proxy=proxy, ipinfo=ipinfo, timeout=timeout,
            allow_redirects=allow_redirects, verify=verify, http_version=http_version, sync=False
        )
        
    async def __aenter__(self):
        return self

    def __enter__(self):
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        pass

    def __exit__(self, exc_type, exc_val, exc_tb):
        pass

    def __await__(self):
        return self
    
    async def update_ipinfo(self):
        try:
            resp = await (await self._session.get("https://ipinfo.io/json", headers={
                "user-agent": self.user_agent,
            }, timeout=5)).json()
            self.ipinfo.update({
                "country": resp["country"].lower(),
                "ip": resp.get("ip"),
                "timezone": resp.get("timezone"),
                "geolocation": resp.get("loc")
            })
            self.client_hints['accept-language'] = ACCEPT_LANGUAGE_MAP.get(self.ipinfo["country"], 'zh-CN,zh;q=0.9')
        except:
            pass

    async def get(self, url: str, *args, **kwargs) -> Response:
        resp = Response(await self._session.get(url, *args, **self._process_get_arguments(**kwargs)))
        self.cookies.update(resp.cookies)
        return resp

    async def post(self, url: str, *args, **kwargs) -> Response:
        resp = Response(await self._session.post(url, *args, **self._process_post_arguments(**kwargs)))
        self.cookies.update(resp.cookies)
        return resp



# -*- coding: UTF-8 -*-

from .base import BaseCracker


class DiscordCracker(BaseCracker):
    
    cracker_name = "discord"
    cracker_version = "guild"    

    """
    discord cracker
    :param authorization: discord 账号登录凭证
    :param guild_id: 群 id
    :param cookies: 
    调用示例:
    cracker = DiscordCracker(
        user_token="xxx",
        authorization="MTExNzI1NDQ3NzA2NjU0MzE5NQ.xxx",
        guild_id='645607528297922560',
        # guild_name='fusionlist'
        # debug=True,
    )
    ret = cracker.crack()
    """
    
    # 必传参数
    must_check_params = ["authorization"]
    # 可选参数
    option_params = {
        "guild_id": "",
        "guild_name": ""
    }



# -*- coding: UTF-8 -*-

import sys

from typing import Optional, Literal, Union, Tuple, Dict, List, Any

if sys.version_info >= (3, 8):  # pragma: no cover (py38+)
    from typing import Literal
else:  # pragma: no cover (py38+)
    from typing_extensions import Literal

from loguru import logger
from curl_cffi import requests

from .base import BaseCracker
from ..magneto.session import AsyncSession, Session
from ..magneto.response import Response


class PerimeterxCracker(BaseCracker):
    
    cracker_name = "perimeterx"
    cracker_version = "universal"    

    """
    perimeterx cracker
    :param tag: px 版本号
    :param href: 触发 perimeterx 验证的页面地址
    :param captcha: 按压验证码参数, 示例: {
        appId: 'PXaOtQIWNf',
        jsClientSrc: '/aOtQIWNf/init.js',
        firstPartyEnabled: true,
        uuid: '013b4cad-ece3-11ee-a877-09542f9a30cf',
        hostUrl: '/aOtQIWNf/xhr',
        blockScript: '/aOtQIWNf/captcha/captcha.js?a=c&u=013b4cad-ece3-11ee-a877-09542f9a30cf&v=&m=0',
        altBlockScript: 'https://captcha.px-cloud.net/PXaOtQIWNf/captcha.js?a=c&u=013b4cad-ece3-11ee-a877-09542f9a30cf&v=&m=0',
        customLogo: 'https://chegg-mobile-promotions.cheggcdn.com/px/Chegg-logo-79X22.png'
    }
    :param user_agent: 请求流程使用 ua, 请使用 chrome 的 ua
    :param headers: 触发验证必须的 headers, 默认 {} 
    :param cookies: 触发验证必须的 cookies, 默认 {}
    :param timeout: 最大破解超时时间
    调用示例:
    cracker = CloudFlareCracker(
        href=href,
        user_token="xxx",
    )
    ret = cracker.crack()
    """
    
    # 必传参数
    must_check_params = ["href"]
    # 默认可选参数
    option_params = {
        "branch": "Master",
        "tag": None,
        "app_id": None,
        "uuid": None,
        "vid": None,
        "modal": False,
        "press": False,
        "captcha": None,
        "captcha_html": None,
        "user_agent": None,
        "did": None,
        "proxy": None,
        "country": None,
        "ip": None,
        "timezone": None,
        "geolocation": None,
        "headers": {},
        "cookies": {},
        "actions": 1,
        "timeout": 30
    }


async def async_crack_perimeterx(
    user_token: str, href: str, app_id: str, press: bool = False, modal: bool = False, pow: bool = False,
    parse_index: Optional[callable] = None, verifiers: List[callable] = [], max_retry_times: int = 2,
    internal_host: bool = True, branch: Optional[str] = None,
    developer_id: Optional[str] = None, auth: bool = False,
    session: Optional[AsyncSession] = None, cookies: Dict[str, str] = {}, proxy: Optional[str] = None,
    timeout: int = 15, allow_redirects: bool = True, verify: bool = False, http_version: Optional[Literal["http1", "http2"]] = None,
    user_agent: Union[str, Tuple[int, Literal["windows", "macos"], Literal["chrome", "edge"]]] = None,
    ipinfo: Dict[str, str] = {}, extra: Dict[str, str] = {}, debug: bool = False
) -> Tuple[AsyncSession, Response, Dict[str, str]]:   
    if not session:
        session = AsyncSession(
            user_agent=user_agent, 
            proxy=proxy, 
            ipinfo=ipinfo, 
            cookies=cookies, 
            timeout=timeout, 
            allow_redirects=allow_redirects, 
            verify=verify, 
            http_version=http_version
        )
        if not ipinfo:
            await session.update_ipinfo()

    if not extra:
        extra.update(session.client_hints)
        
    user_agent = session.user_agent
    proxy = session.proxy
    
    nocaptcha_host = "api.nocaptcha.cn" if internal_host else "api.nocaptcha.io"
    
    async with requests.AsyncSession() as _session:
        
        headers = {
            'sec-ch-ua': session.client_hints['sec-ch-ua'], 
            'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7', 
            'upgrade-insecure-requests': '1', 
            'sec-ch-ua-mobile': '?0', 
            'user-agent': user_agent, 
            'sec-ch-ua-platform': session.client_hints['sec-ch-ua-platform'],
            'Sec-Fetch-Site': 'none', 
            'Sec-Fetch-Mode': 'navigate', 
            'Sec-Fetch-User': '?1',
            'Sec-Fetch-Dest': 'document',
            'accept-encoding': 'gzip, deflate, br, zstd',
            'accept-language': session.client_hints['accept-language'], 
            'priority': 'u=0, i'
        }

        resp = await session.get(href, headers=headers)
        
        retry_times = 0
        while resp.status_code == 403 and retry_times < max_retry_times:
            if debug:
                logger.debug("触发按压模式")
                
            nocaptcha_resp = (await _session.post(
                f'http://{nocaptcha_host}/api/wanda/perimeterx/universal', headers={
                    'user-token': user_token,
                    **({ "Developer-Id": developer_id } if developer_id else {})
                }, json={
                    'is_auth': auth,
                    "branch": branch,
                    "href": href,
                    "app_id": app_id,
                    "captcha_html": await resp.async_text(),
                    "cookies": session.cookies,
                    "user_agent": user_agent,
                    "proxy": proxy,
                    **session.ipinfo,
                },
                timeout=120
            )).json()
            if debug:
                logger.debug(nocaptcha_resp)

            if nocaptcha_resp["status"]:
                extra.update(nocaptcha_resp["extra"])
                session.cookies.update(nocaptcha_resp["data"]['cookies'])
                if parse_index:
                    resp = await session.get(href, headers=headers)
                    retry_times += 1
                else:
                    break
            else:
                raise Warning(f'验证失败, id: {nocaptcha_resp["id"]}, err: {nocaptcha_resp["msg"]}')
        
        if retry_times == 0:
            nocaptcha_resp = (await _session.post(
                f'http://{nocaptcha_host}/api/wanda/perimeterx/universal', headers={
                    'user-token': user_token,
                    **({ "Developer-Id": developer_id } if developer_id else {})
                }, json={
                    'is_auth': auth,
                    "branch": branch,
                    "href": href,
                    "app_id": app_id,
                    "press": press,
                    "modal": modal,
                    "pow": pow,
                    "cookies": session.cookies,
                    "user_agent": user_agent,
                    "proxy": proxy,
                    **session.ipinfo,
                },
                timeout=120
            )).json()
            if debug:
                logger.debug(nocaptcha_resp)

            if nocaptcha_resp["status"]:
                extra.update(nocaptcha_resp["extra"])
                session.cookies.update(nocaptcha_resp["data"]['cookies'])
            else:
                raise Warning(f'验证失败, id: {nocaptcha_resp["id"]}, err: {nocaptcha_resp["msg"]}')

        if parse_index:
            await parse_index(resp, extra)
        
        for verifier in verifiers:            
            resp = await verifier(session, extra)
                
            retry_times = 0
            while resp.status_code == 403 and retry_times < max_retry_times:
                did = extra.get("did")
                _branch = extra.get("branch")
                if _branch:
                    branch = _branch
                    
                captcha_args = {}
                try:
                    captcha_args["captcha"] = await resp.async_json()
                except:
                    captcha_args["captcha_html"] = await resp.async_text()
                
                nocaptcha_resp = (await _session.post(
                    f'http://{nocaptcha_host}/api/wanda/perimeterx/universal', headers={
                        'user-token': user_token,
                        **({ "Developer-Id": developer_id } if developer_id else {})
                    }, json={
                        'is_auth': auth,
                        "branch": branch,
                        "href": href,
                        "app_id": app_id,
                        **captcha_args,
                        "did": did,
                        "cookies": session.cookies,
                        "user_agent": user_agent,
                        "proxy": proxy,
                        **session.ipinfo,
                    },
                    timeout=120
                )).json()
                if debug:
                    logger.debug(f'验证结果: {nocaptcha_resp}')
                
                if nocaptcha_resp["status"]:
                    extra.update(nocaptcha_resp["extra"])
                    session.cookies.update(nocaptcha_resp['data']['cookies'])
                    resp = await verifier(session, extra)
                else:
                    raise Warning(f'验证失败, id: {resp["id"]}, err: {resp["msg"]}')
                
                retry_times += 1

        return session, resp, extra


def crack_perimeterx(
    user_token: str, href: str, app_id: str, press: bool = False, modal: bool = False, pow: bool = False, 
    parse_index: Optional[callable] = None, verifiers: List[callable] = [], max_retry_times: int = 2,
    internal_host: bool = True, branch: Optional[str] = None,
    developer_id: Optional[str] = None, auth: bool = False,
    session: Optional[AsyncSession] = None, cookies: Dict[str, str] = {}, proxy: Optional[str] = None,
    timeout: int = 15, allow_redirects: bool = True, verify: bool = False, http_version: Optional[Literal["http1", "http2"]] = None,
    user_agent: Union[str, Tuple[int, Literal["windows", "macos"], Literal["chrome", "edge"]]] = None,
    ipinfo: Dict[str, str] = {}, extra: Dict[str, str] = {}, debug: bool = False
) -> Tuple[Session, Response, Dict[str, str]]:   
    if not session:
        session = Session(
            user_agent=user_agent, 
            proxy=proxy, 
            ipinfo=ipinfo, 
            cookies=cookies, 
            timeout=timeout, 
            allow_redirects=allow_redirects, 
            verify=verify, 
            http_version=http_version
        )
        if not ipinfo:
            session.update_ipinfo()

    if not extra:
        extra.update(session.client_hints)
        
    user_agent = session.user_agent
    proxy = session.proxy
    
    nocaptcha_host = "api.nocaptcha.cn" if internal_host else "api.nocaptcha.io"
    
    headers = {
        'sec-ch-ua': session.client_hints['sec-ch-ua'], 
        'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7', 
        'upgrade-insecure-requests': '1', 
        'sec-ch-ua-mobile': '?0', 
        'user-agent': user_agent, 
        'sec-ch-ua-platform': session.client_hints['sec-ch-ua-platform'],
        'Sec-Fetch-Site': 'none', 
        'Sec-Fetch-Mode': 'navigate', 
        'Sec-Fetch-User': '?1',
        'Sec-Fetch-Dest': 'document',
        'accept-encoding': 'gzip, deflate, br, zstd',
        'accept-language': session.client_hints['accept-language'], 
        'priority': 'u=0, i'
    }

    resp = session.get(href, headers=headers)
    
    retry_times = 0
    while resp.status_code == 403 and retry_times < max_retry_times:
        if debug:
            logger.debug("触发按压模式")
            
        nocaptcha_resp = requests.post(
            f'http://{nocaptcha_host}/api/wanda/perimeterx/universal', headers={
                'user-token': user_token,
                **({ "Developer-Id": developer_id } if developer_id else {})
            }, json={
                'is_auth': auth,
                "branch": branch,
                "href": href,
                "app_id": app_id,
                "captcha_html": resp.text,
                "cookies": session.cookies,
                "user_agent": user_agent,
                "proxy": proxy,
                **session.ipinfo,
            },
            timeout=120
        ).json()
        if debug:
            logger.debug(nocaptcha_resp)

        if nocaptcha_resp["status"]:
            extra.update(nocaptcha_resp["extra"])
            session.cookies.update(nocaptcha_resp["data"]['cookies'])
            if parse_index:
                resp = session.get(href, headers=headers)
                retry_times += 1
            else:
                break
        else:
            raise Warning(f'验证失败, id: {nocaptcha_resp["id"]}, err: {nocaptcha_resp["msg"]}')
    
    if retry_times == 0:
        nocaptcha_resp = requests.post(
            f'http://{nocaptcha_host}/api/wanda/perimeterx/universal', headers={
                'user-token': user_token,
                **({ "Developer-Id": developer_id } if developer_id else {})
            }, json={
                'is_auth': auth,
                "branch": branch,
                "href": href,
                "app_id": app_id,
                "press": press,
                "modal": modal,
                "pow": pow,
                "cookies": session.cookies,
                "user_agent": user_agent,
                "proxy": proxy,
                **session.ipinfo,
            },
            timeout=120
        ).json()
        if debug:
            logger.debug(nocaptcha_resp)

        if nocaptcha_resp["status"]:
            extra.update(nocaptcha_resp["extra"])
            session.cookies.update(nocaptcha_resp["data"]['cookies'])
        else:
            raise Warning(f'验证失败, id: {nocaptcha_resp["id"]}, err: {nocaptcha_resp["msg"]}')

    if parse_index:
        parse_index(resp, extra)
    
    for verifier in verifiers:            
        resp = verifier(session, extra)
            
        retry_times = 0
        while resp.status_code == 403 and retry_times < max_retry_times:
            did = extra.get("did")
            _branch = extra.get("branch")
            if _branch:
                branch = _branch
                    
            captcha_args = {}
            try:
                captcha_args["captcha"] = resp.json()
            except:
                captcha_args["captcha_html"] = resp.text
            
            nocaptcha_resp = requests.post(
                f'http://{nocaptcha_host}/api/wanda/perimeterx/universal', headers={
                    'user-token': user_token,
                    **({ "Developer-Id": developer_id } if developer_id else {})
                }, json={
                    'is_auth': auth,
                    "branch": branch,
                    "href": href,
                    "app_id": app_id,
                    **captcha_args,
                    "did": did,
                    "cookies": session.cookies,
                    "user_agent": user_agent,
                    "proxy": proxy,
                    **session.ipinfo,
                },
                timeout=120
            ).json()
            if debug:
                logger.debug(nocaptcha_resp)
            
            if nocaptcha_resp['status']:
                session.cookies.update(nocaptcha_resp['data']['cookies'])
                resp = verifier(session, extra)
            else:
                raise Warning(f'验证失败, id: {nocaptcha_resp["id"]}, err: {nocaptcha_resp["msg"]}')
            
            retry_times += 1

    return session, resp, extra



# -*- coding: UTF-8 -*-

import sys
import re
import base64
from curl_cffi import requests
from loguru import logger

from typing import Union, Tuple, Optional, Dict, Any

if sys.version_info >= (3, 8):  # pragma: no cover (py38+)
    from typing import Literal
else:  # pragma: no cover (py38+)
    from typing_extensions import Literal

from .base import BaseCracker
from ..magneto.session import AsyncSession, Session


class KasadaCdCracker(BaseCracker):
    cracker_name = "kasada"
    cracker_version = "cd"

    """
    kasada x-kpsdk-ct cracker
    :param href: 触发验证的页面地址
    调用示例:
    cracker = KasadaCdCracker(
        user_token="xxx",
        href="https://arcteryx.com/ca/en/shop/mens/beta-lt-jacket-7301",
        debug=True,
    )
    ret = cracker.crack()
    """

    # 必传参数
    must_check_params = ["href", "st"]
    option_params = {
        "ct": ""
    }


async def async_crack_kasada(
    user_token: str, href: str, proxy: Optional[str] = None, kpsdk_v: Optional[str] = None, fp_host: Optional[str] = None, iframe: bool = True, submit: bool = False,
    internal_host: bool = True, branch: Optional[str] = None,
    developer_id: Optional[str] = None, auth: bool = False,
    session: Optional[AsyncSession] = None, cookies: Dict[str, str] = {},
    timeout: int = 15, allow_redirects: bool = True, verify: bool = False, http_version: Optional[Literal["http1", "http2"]] = None,
    user_agent: Union[str, Tuple[int, Literal["windows", "macos"], Literal["chrome", "edge"]]] = None,
    ipinfo: Dict[str, str] = {}, extra: Dict[str, str] = {}, debug: bool = False
) -> Tuple[AsyncSession, Dict[str, Any], Dict[str, Any]]:
    if not session:
        session = AsyncSession(
            user_agent=user_agent, 
            proxy=proxy, 
            ipinfo=ipinfo, 
            cookies=cookies, 
            timeout=timeout, 
            allow_redirects=allow_redirects, 
            verify=verify, 
            http_version=http_version
        )
        if not ipinfo:
            await session.update_ipinfo()

    if not extra:
        extra.update(session.client_hints)
    
    user_agent = session.user_agent
    proxy = session.proxy

    headers = {
        'sec-ch-ua': session.client_hints['sec-ch-ua'],
        'sec-ch-ua-mobile': '?0',
        'sec-ch-ua-platform': session.client_hints['sec-ch-ua-platform'],
        'upgrade-insecure-requests': '1',
        'user-agent': user_agent,
        'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7',
        'sec-fetch-site': 'none',
        'sec-fetch-mode': 'navigate',
        'sec-fetch-dest': 'document',
        'referer': href,
        'accept-encoding': 'gzip, deflate, br, zstd',
        'accept-language': session.client_hints['accept-language'],
        'priority': 'u=0, i'
    }
    
    origin = "/".join(href.split("/")[0:3])
    _fp_protocol = href.split("://")[0]
    if not fp_host:
        fp_host = href.split("/")[2]
        
    if not kpsdk_v:
        if '?x-kpsdk-v=' in href:
            kpsdk_v = href.split("?x-kpsdk-v=")[1]
    
    if iframe:            
        headers["sec-fetch-dest"] = "iframe"
        headers["sec-fetch-site"] = "same-site"
        response = await session.get(f"{_fp_protocol}://{fp_host}/149e9513-01fa-4fb0-aad4-566afd725d1b/2d206a39-8ed7-437e-a3be-862e0f06eea3/fp?x-kpsdk-v={kpsdk_v}", headers=headers)
    else:
        response = await session.get(href, headers=headers)

    if response.status_code not in [429, 200]:
        raise Exception(f"fp status code: {response.status_code}")
    
    fp_html = await response.async_text()
    if '/149e9513-01fa-4fb0-aad4-566afd725d1b/2d206a39-8ed7-437e-a3be-862e0f06eea3/ips.js' not in fp_html and 'KPSDK' in fp_html:
        kpsdk_args = {
            "href": href,
            "fp_host": fp_host,
            "fp_html": fp_html,
            "user_agent": user_agent,
            "cookies": session.cookies,
            "iframe": iframe,
        }
    else:            
        # KP_UIDz
        uidk = None
        for k, _ in response.cookies.items():
            if k.endswith('-ssn'):
                uidk = k.replace('-ssn', '')
        
        if not uidk:
            raise Warning("站点异常, 请联系管理员处理")

        ips_url = _fp_protocol + '://' + fp_host + re.search(r'src="(\/149e9513-01fa-4fb0-aad4-566afd725d1b\/2d206a39-8ed7-437e-a3be-862e0f06eea3\/ips\.js\?.*?)"', fp_html)[1].replace('amp;', '')
        
        headers = {
            'sec-ch-ua-platform': session.client_hints['sec-ch-ua-platform'],
            'user-agent': user_agent,
            'sec-ch-ua': session.client_hints['sec-ch-ua'],
            'sec-ch-ua-mobile': '?0',
            'accept': '*/*',
            'sec-fetch-site': 'same-origin',
            'sec-fetch-mode': 'no-cors',
            'sec-fetch-dest': 'script',
            'referer': f"{_fp_protocol}://{fp_host}/149e9513-01fa-4fb0-aad4-566afd725d1b/2d206a39-8ed7-437e-a3be-862e0f06eea3/fp?x-kpsdk-v={kpsdk_v}",
            'accept-encoding': 'gzip, deflate, br, zstd',
            'accept-language': session.client_hints['accept-language'],
            'priority': 'u=1'
        }
        ips_resp = await session.get(ips_url, headers=headers)
        kpsdk_args = {
            "href": href,
            "fp_host": fp_host,
            "fp_html": fp_html,
            "ips_script": await ips_resp.async_text(),
            "ips_headers": ips_resp.headers,
            "cookies": session.cookies,
            "user_agent": user_agent,
            "iframe": iframe,
        }
    
    kpsdk_args.update({
        **session.ipinfo,
        "submit": submit,
        "branch": branch,
        'is_auth': auth,
    })

    if submit:
        kpsdk_args['proxy'] = proxy
    
    nocaptcha_host = "api.nocaptcha.cn" if internal_host else "api.nocaptcha.io"
    async with requests.AsyncSession() as _session:
        nocaptcha_resp = (await _session.post(
            f'http://{nocaptcha_host}/api/wanda/kasada/ct', headers={
                'user-token': user_token,
                **({ "Developer-Id": developer_id } if developer_id else {})
            }, json=kpsdk_args
        )).json()
    
        if nocaptcha_resp["status"]:
            extra = nocaptcha_resp['extra']
            if not submit:
                extra = nocaptcha_resp['extra']
                post_data = base64.b64decode(nocaptcha_resp["data"]["post_data"].encode())
                headers = {
                    'content-length': str(len(post_data)),
                    'x-kpsdk-ct': nocaptcha_resp["data"]["headers"]['x-kpsdk-ct'],
                    'sec-ch-ua-platform': extra['sec-ch-ua-platform'],
                    'x-kpsdk-dt': nocaptcha_resp["data"]["headers"]['x-kpsdk-dt'],
                    'sec-ch-ua': extra['sec-ch-ua'],
                    'x-kpsdk-im': nocaptcha_resp["data"]["headers"]['x-kpsdk-im'],
                    'sec-ch-ua-mobile': '?0',
                    'x-kpsdk-v': nocaptcha_resp["data"]["headers"].get('x-kpsdk-v') or kpsdk_v,
                    'user-agent': extra['user-agent'],
                    'content-type': 'application/octet-stream',
                    'accept': '*/*',
                    'origin': origin,
                    'sec-fetch-site': 'same-origin',
                    'sec-fetch-mode': 'cors',
                    'sec-fetch-dest': 'empty',
                    'referer': f"{_fp_protocol}://{fp_host}/149e9513-01fa-4fb0-aad4-566afd725d1b/2d206a39-8ed7-437e-a3be-862e0f06eea3/fp?x-kpsdk-v={kpsdk_v}" if iframe else href,
                    'accept-encoding': 'gzip, deflate, br, zstd',
                    'accept-language': extra['accept-language'],
                    'priority': 'u=1, i',
                }

                response = await session.post(
                    f'{_fp_protocol}://{fp_host}/149e9513-01fa-4fb0-aad4-566afd725d1b/2d206a39-8ed7-437e-a3be-862e0f06eea3/tl',
                    headers=headers,
                    body=base64.b64decode(nocaptcha_resp["data"]["post_data"].encode()),
                )
                if 'reload' in await response.async_text():
                    kpsdk_ct = response.headers.get('x-kpsdk-ct')
                    kpsdk_st = int(response.headers.get('x-kpsdk-st'))
                    kpsdk_res = {
                        'x-kpsdk-ct': kpsdk_ct,
                        'x-kpsdk-st': kpsdk_st
                    }
                else:
                    raise Warning("验证失败")
            else:
                kpsdk_res = nocaptcha_resp['data']
            
            kpsdk_res.update({
                'x-kpsdk-v': kpsdk_v,
            })
            if debug:
                logger.debug(kpsdk_res)
            return session, kpsdk_res, extra
        else:
            raise Warning("计算失败")


def crack_kasada(
    user_token: str, href: str, proxy: Optional[str] = None, kpsdk_v: Optional[str] = None, fp_host: Optional[str] = None, iframe: bool = True, submit: bool = False,
    internal_host: bool = True, branch: Optional[str] = None,
    developer_id: Optional[str] = None, auth: bool = False,
    session: Optional[Session] = None, cookies: Dict[str, str] = {},
    timeout: int = 15, allow_redirects: bool = True, verify: bool = False, http_version: Optional[Literal["http1", "http2"]] = None,
    user_agent: Union[str, Tuple[int, Literal["windows", "macos"], Literal["chrome", "edge"]]] = None,
    ipinfo: Dict[str, str] = {}, extra: Dict[str, str] = {}, debug: bool = False
) -> Tuple[Session, Dict[str, Any], Dict[str, Any]]:
    if not session:
        session = Session(
            user_agent=user_agent, 
            proxy=proxy, 
            ipinfo=ipinfo, 
            cookies=cookies, 
            timeout=timeout, 
            allow_redirects=allow_redirects, 
            verify=verify, 
            http_version=http_version
        )
        if not ipinfo:
            session.update_ipinfo()

    if not extra:
        extra.update(session.client_hints)
        
    user_agent = session.user_agent
    proxy = session.proxy
    
    headers = {
        'sec-ch-ua': session.client_hints['sec-ch-ua'],
        'sec-ch-ua-mobile': '?0',
        'sec-ch-ua-platform': session.client_hints['sec-ch-ua-platform'],
        'upgrade-insecure-requests': '1',
        'user-agent': user_agent,
        'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7',
        'sec-fetch-site': 'none',
        'sec-fetch-mode': 'navigate',
        'sec-fetch-dest': 'document',
        'referer': href,
        'accept-encoding': 'gzip, deflate, br, zstd',
        'accept-language': session.client_hints['accept-language'],
        'priority': 'u=0, i'
    }
    
    origin = "/".join(href.split("/")[0:3])
    _fp_protocol = href.split("://")[0]
    if not fp_host:
        fp_host = href.split("/")[2]
        
    if not kpsdk_v:
        if '?x-kpsdk-v=' in href:
            kpsdk_v = href.split("?x-kpsdk-v=")[1]
    
    if iframe:            
        headers["sec-fetch-dest"] = "iframe"
        headers["sec-fetch-site"] = "same-site"
        response = session.get(f"{_fp_protocol}://{fp_host}/149e9513-01fa-4fb0-aad4-566afd725d1b/2d206a39-8ed7-437e-a3be-862e0f06eea3/fp?x-kpsdk-v={kpsdk_v}", headers=headers)
    else:
        response = session.get(href, headers=headers)

    if response.status_code not in [429, 200]:
        raise Exception(f"fp status code: {response.status_code}")
    
    fp_html = response.text
    if '/149e9513-01fa-4fb0-aad4-566afd725d1b/2d206a39-8ed7-437e-a3be-862e0f06eea3/ips.js' not in fp_html and 'KPSDK' in fp_html:
        kpsdk_args = {
            "href": href,
            "fp_host": fp_host,
            "fp_html": fp_html,
            "user_agent": user_agent,
            "cookies": session.cookies,
            "iframe": iframe,
        }
    else:            
        # KP_UIDz
        uidk = None
        for k, _ in response.cookies.items():
            if k.endswith('-ssn'):
                uidk = k.replace('-ssn', '')
        
        if not uidk:
            raise Warning("站点异常, 请联系管理员处理")

        ips_url = _fp_protocol + '://' + fp_host + re.search(r'src="(\/149e9513-01fa-4fb0-aad4-566afd725d1b\/2d206a39-8ed7-437e-a3be-862e0f06eea3\/ips\.js\?.*?)"', fp_html)[1].replace('amp;', '')
        
        headers = {
            'sec-ch-ua-platform': session.client_hints['sec-ch-ua-platform'],
            'user-agent': user_agent,
            'sec-ch-ua': session.client_hints['sec-ch-ua'],
            'sec-ch-ua-mobile': '?0',
            'accept': '*/*',
            'sec-fetch-site': 'same-origin',
            'sec-fetch-mode': 'no-cors',
            'sec-fetch-dest': 'script',
            'referer': f"{_fp_protocol}://{fp_host}/149e9513-01fa-4fb0-aad4-566afd725d1b/2d206a39-8ed7-437e-a3be-862e0f06eea3/fp?x-kpsdk-v={kpsdk_v}",
            'accept-encoding': 'gzip, deflate, br, zstd',
            'accept-language': session.client_hints['accept-language'],
            'priority': 'u=1'
        }
        ips_resp = session.get(ips_url, headers=headers)
        kpsdk_args = {
            "href": href,
            "fp_host": fp_host,
            "fp_html": fp_html,
            "ips_script": ips_resp.text,
            "ips_headers": ips_resp.headers,
            "cookies": session.cookies,
            "user_agent": user_agent,
            "iframe": iframe,
        }
    
    kpsdk_args.update({
        **session.ipinfo,
        "submit": submit,
        "branch": branch,
        'is_auth': auth,
    })

    if submit:
        kpsdk_args['proxy'] = proxy
    
    nocaptcha_host = "api.nocaptcha.cn" if internal_host else "api.nocaptcha.io"
    nocaptcha_resp = requests.post(
        f'http://{nocaptcha_host}/api/wanda/kasada/ct', headers={
            'user-token': user_token,
            **({ "Developer-Id": developer_id } if developer_id else {})
        }, json=kpsdk_args
    ).json()
    
    if nocaptcha_resp["status"]:
        if not submit:
            extra.update(nocaptcha_resp['extra'])
            post_data = base64.b64decode(nocaptcha_resp["data"]["post_data"].encode())
            headers = {
                'content-length': str(len(post_data)),
                'x-kpsdk-ct': nocaptcha_resp["data"]["headers"]['x-kpsdk-ct'],
                'sec-ch-ua-platform': extra['sec-ch-ua-platform'],
                'x-kpsdk-dt': nocaptcha_resp["data"]["headers"]['x-kpsdk-dt'],
                'sec-ch-ua': extra['sec-ch-ua'],
                'x-kpsdk-im': nocaptcha_resp["data"]["headers"]['x-kpsdk-im'],
                'sec-ch-ua-mobile': '?0',
                'x-kpsdk-v': nocaptcha_resp["data"]["headers"].get('x-kpsdk-v') or kpsdk_v,
                'user-agent': extra['user-agent'],
                'content-type': 'application/octet-stream',
                'accept': '*/*',
                'origin': origin,
                'sec-fetch-site': 'same-origin',
                'sec-fetch-mode': 'cors',
                'sec-fetch-dest': 'empty',
                'referer': f"{_fp_protocol}://{fp_host}/149e9513-01fa-4fb0-aad4-566afd725d1b/2d206a39-8ed7-437e-a3be-862e0f06eea3/fp?x-kpsdk-v={kpsdk_v}" if iframe else href,
                'accept-encoding': 'gzip, deflate, br, zstd',
                'accept-language': extra['accept-language'],
                'priority': 'u=1, i',
            }

            response = session.post(
                f'{_fp_protocol}://{fp_host}/149e9513-01fa-4fb0-aad4-566afd725d1b/2d206a39-8ed7-437e-a3be-862e0f06eea3/tl',
                headers=headers,
                body=base64.b64decode(nocaptcha_resp["data"]["post_data"].encode()),
            )
            if 'reload' in response.text:
                kpsdk_ct = response.headers.get('x-kpsdk-ct')
                kpsdk_st = int(response.headers.get('x-kpsdk-st'))
                kpsdk_res = {
                    'x-kpsdk-ct': kpsdk_ct,
                    'x-kpsdk-st': kpsdk_st
                }
            else:
                raise Warning("验证失败")
        else:
            kpsdk_res = nocaptcha_resp['data']
        
        kpsdk_res.update({
            'x-kpsdk-v': kpsdk_v,
        })
        if debug:
            logger.debug(kpsdk_res)
        
        return session, kpsdk_res, extra
    else:
        raise Warning(f'计算失败, id: {nocaptcha_resp["id"]}, err: {nocaptcha_resp["msg"]}')



# -*- coding: UTF-8 -*-

import re
import sys

from typing import Optional, List, Dict, Any, Tuple, Union

if sys.version_info >= (3, 8):  # pragma: no cover (py38+)
    from typing import Literal
else:  # pragma: no cover (py38+)
    from typing_extensions import Literal

import json
from loguru import logger
from curl_cffi import requests

from ..magneto.session import Session, AsyncSession
from ..magneto.response import Response


def crack_datadome(
    user_token: str, href: str, interstitial: bool = False,
    js_url: Optional[str] = None, js_key: Optional[str] = None, 
    parse_index: Optional[callable] = None, verifiers: List[callable] = [], max_retry_times: int = 2, 
    internal_host: bool = True, branch: Optional[str] = None,
    developer_id: Optional[str] = None, auth: bool = False,
    session: Optional[Session] = None, cookies: Dict[str, str] = {}, proxy: Optional[str] = None,
    timeout: int = 15, allow_redirects: bool = True, verify: bool = False, http_version: Optional[Literal["http1", "http2"]] = None,
    user_agent: Union[str, Tuple[int, Literal["windows", "macos"], Literal["chrome", "edge"]]] = None,
    ipinfo: Dict[str, str] = {}, extra: Dict[str, str] = {}, debug: bool = False
) -> Tuple[Session, Response, Dict[str, Any]]:    
    if not session:
        session = Session(
            user_agent=user_agent, 
            proxy=proxy, 
            ipinfo=ipinfo, 
            cookies=cookies, 
            timeout=timeout, 
            allow_redirects=allow_redirects, 
            verify=verify, 
            http_version=http_version
        )
        if not ipinfo:
            session.update_ipinfo()

    if not extra:
        extra.update(session.client_hints)
        
    user_agent = session.user_agent
    proxy = session.proxy
    
    nocaptcha_host = "api.nocaptcha.cn" if internal_host else "api.nocaptcha.io"

    if interstitial:
        html = parse_index is None

        nocaptcha_resp = requests.post(
            f'http://{nocaptcha_host}/api/wanda/datadome/universal', headers={
                'user-token': user_token,
                **({ "Developer-Id": developer_id } if developer_id else {})
            }, json={
                'is_auth': auth,
                "branch": branch,
                "href": href,
                "interstitial": interstitial,
                "html": html,
                "user_agent": user_agent,
                "proxy": proxy,
                **session.ipinfo,
            }
        ).json()
        if debug:
            logger.debug(nocaptcha_resp)

        if nocaptcha_resp["status"]:
            extra = nocaptcha_resp["extra"]
            session.cookies.update(nocaptcha_resp["data"])
        else:
            raise Warning(f'验证失败, id: {nocaptcha_resp["id"]}, err: {nocaptcha_resp["msg"]}')
    else:
        headers = {
            'sec-ch-ua': session.client_hints['sec-ch-ua'], 
            'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7', 
            'upgrade-insecure-requests': '1', 
            'sec-ch-ua-mobile': '?0', 
            'user-agent': user_agent, 
            'sec-ch-ua-platform': session.client_hints['sec-ch-ua-platform'],
            'Sec-Fetch-Site': 'none', 
            'Sec-Fetch-Mode': 'navigate', 
            'Sec-Fetch-User': '?1',
            'Sec-Fetch-Dest': 'document',
            'accept-encoding': 'gzip, deflate, br, zstd',
            'accept-language': session.client_hints['accept-language'], 
            'priority': 'u=0, i'
        }

        resp = session.get(href, headers=headers)
        html = resp.text
        
        retry_times = 0
        while resp.status_code in [403, 405] and retry_times < max_retry_times:
            try:
                dd_match = re.search(r'var dd=(\{.*?\})', html)
                captcha = json.loads(dd_match[1].replace("'", '"'))
                if captcha.get("t") == "bv":
                    raise Warning("代理被封锁, 请切换代理重试")
            except:
                raise Warning(f"验证异常: {resp.status_code} {html}")

            if debug:
                logger.debug(f'触发验证码: {html}')
            
            datadome = resp.cookies.get('datadome')

            nocaptcha_resp = requests.post(
                f'http://{nocaptcha_host}/api/wanda/datadome/universal', headers={
                    'user-token': user_token,
                    **({ "Developer-Id": developer_id } if developer_id else {})
                }, json={
                    'is_auth': auth,
                    "branch": branch,
                    "href": href,
                    "captcha_html": html,
                    "cookies": {
                        'datadome': datadome
                    },
                    "html": html,
                    "user_agent": user_agent,
                    "proxy": proxy,
                    **session.ipinfo,
                }
            ).json()
            if debug:
                logger.debug(nocaptcha_resp)
            
            if nocaptcha_resp["status"]:
                extra = nocaptcha_resp["extra"]
                session.cookies.update(nocaptcha_resp["data"])
                if parse_index:
                    resp = session.get(href, headers=headers)
                    retry_times += 1
                else:
                    break            
            else:
                raise Warning(f'验证失败, id: {nocaptcha_resp["id"]}, err: {nocaptcha_resp["msg"]}')
            
        if retry_times == 0 and js_url and js_key:
            datadome = session.cookies.get('datadome')
            _cookies = None
            if datadome:
                _cookies = {
                    "datadome": datadome
                }
            
            nocaptcha_resp = requests.post(
                f'http://{nocaptcha_host}/api/wanda/datadome/universal', headers={
                    'user-token': user_token,
                    **({ "Developer-Id": developer_id } if developer_id else {})
                }, json={
                    'is_auth': auth,
                    "branch": branch,
                    "href": href,
                    "js_url": js_url,
                    "js_key": js_key,
                    "cookies": _cookies,
                    "user_agent": user_agent,
                    "proxy": proxy,
                    **session.ipinfo,
                }
            ).json()
            if debug:
                logger.debug(nocaptcha_resp)
                
            if nocaptcha_resp["status"]:
                extra.update(nocaptcha_resp['extra'])
                session.cookies.update(nocaptcha_resp['data'])
            else:
                raise Warning(f'验证失败, id: {nocaptcha_resp["id"]}, err: {nocaptcha_resp["msg"]}')
    
    if parse_index:
        if interstitial:
            parse_index(extra["html"])
        else:
            parse_index(resp, extra)
    
    for verifier in verifiers:
        
        retry_times = 0
        
        resp = verifier(session, extra)

        while resp.status_code in [403, 405] and retry_times < max_retry_times:
            did = extra.get("did")
            
            datadome = session.cookies["datadome"]
            
            captcha_args = {}
            try:
                captcha_url = resp.json()["url"]
                if 't=bv' in captcha_url:
                    raise Warning("代理被封锁, 请切换代理重试")
                captcha_args["captcha_url"] = captcha_url
            except Warning as e:
                raise e
            except:
                captcha_html = resp.text
                try:
                    dd_match = re.search(r'var dd=(\{.*?\})', captcha_html)
                    captcha = json.loads(dd_match[1].replace("'", '"'))
                    if captcha.get("t") == "bv":
                        raise Warning("代理被封锁, 请切换代理重试")
                except:
                    raise Warning("验证异常: " + captcha_html)
                
                captcha_args["captcha_html"] = captcha_html
            
            if debug:
                logger.debug(f'触发验证码: {captcha_args}')
                
            nocaptcha_resp = requests.post(
                f'http://{nocaptcha_host}/api/wanda/datadome/universal', headers={
                    'user-token': user_token,
                    **({ "Developer-Id": developer_id } if developer_id else {})
                }, json={
                    'is_auth': auth,
                    "branch": branch,
                    "href": href,
                    **captcha_args,
                    "did": did,
                    "cookies": {
                        "datadome": datadome
                    },
                    "user_agent": user_agent,
                    "proxy": proxy,
                    **session.ipinfo,
                }
            ).json()
            if debug:
                logger.debug(nocaptcha_resp)
                
            if nocaptcha_resp["status"]:
                session.cookies.update(nocaptcha_resp['data'])
                resp = verifier(session, extra)
            else:
                raise Warning(f'验证失败, id: {nocaptcha_resp["id"]}, err: {nocaptcha_resp["msg"]}')
            
            retry_times += 1

    return session, resp, extra


async def async_crack_datadome(
    user_token: str, href: str, interstitial: bool = False,
    js_url: Optional[str] = None, js_key: Optional[str] = None, 
    parse_index: Optional[callable] = None, verifiers: List[callable] = [], max_retry_times: int = 2, 
    internal_host: bool = True, branch: Optional[str] = None,
    developer_id: Optional[str] = None, auth: bool = False,
    session: Optional[AsyncSession] = None, cookies: Dict[str, str] = {}, proxy: Optional[str] = None,
    timeout: int = 15, allow_redirects: bool = True, verify: bool = False, http_version: Optional[Literal["http1", "http2"]] = None,
    user_agent: Union[str, Tuple[int, Literal["windows", "macos"], Literal["chrome", "edge"]]] = None,
    ipinfo: Dict[str, str] = {}, extra: Dict[str, str] = {}, debug: bool = False
) -> Tuple[AsyncSession, Response, Dict[str, Any]]:    
    if not session:
        session = AsyncSession(
            user_agent=user_agent, 
            proxy=proxy, 
            ipinfo=ipinfo, 
            cookies=cookies, 
            timeout=timeout, 
            allow_redirects=allow_redirects, 
            verify=verify, 
            http_version=http_version
        )
        if not ipinfo:
            await session.update_ipinfo()

    if not extra:
        extra.update(session.client_hints)
        
    user_agent = session.user_agent
    proxy = session.proxy
    
    nocaptcha_host = "api.nocaptcha.cn" if internal_host else "api.nocaptcha.io"
    
    async with requests.AsyncSession() as _session:
        resp = None
        if interstitial:
            html = parse_index is None

            nocaptcha_resp = (await _session.post(
                f'http://{nocaptcha_host}/api/wanda/datadome/universal', headers={
                    'user-token': user_token,
                    **({ "Developer-Id": developer_id } if developer_id else {})
                }, json={
                    'is_auth': auth,
                    "branch": branch,
                    "href": href,
                    "interstitial": interstitial,
                    "html": html,
                    "user_agent": user_agent,
                    "proxy": proxy,
                    **session.ipinfo,
                }
            )).json()
            if debug:
                logger.debug(nocaptcha_resp)

            if nocaptcha_resp["status"]:
                extra = nocaptcha_resp["extra"]
                session.cookies.update(nocaptcha_resp["data"])
            else:
                raise Warning(f'验证失败, id: {nocaptcha_resp["id"]}, err: {nocaptcha_resp["msg"]}')
        else:
            headers = {
                'sec-ch-ua': session.client_hints['sec-ch-ua'], 
                'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7', 
                'upgrade-insecure-requests': '1', 
                'sec-ch-ua-mobile': '?0', 
                'user-agent': user_agent, 
                'sec-ch-ua-platform': session.client_hints['sec-ch-ua-platform'],
                'Sec-Fetch-Site': 'none', 
                'Sec-Fetch-Mode': 'navigate', 
                'Sec-Fetch-User': '?1',
                'Sec-Fetch-Dest': 'document',
                'accept-encoding': 'gzip, deflate, br, zstd',
                'accept-language': session.client_hints['accept-language'], 
                'priority': 'u=0, i'
            }

            resp = await session.get(href, headers=headers)
            html = await resp.async_text()
            
            retry_times = 0
            while resp.status_code in [403, 405] and retry_times < max_retry_times:
                try:
                    dd_match = re.search(r'var dd=(\{.*?\})', html)
                    captcha = json.loads(dd_match[1].replace("'", '"'))
                    if captcha.get("t") == "bv":
                        raise Warning("代理被封锁, 请切换代理重试")
                except:
                    raise Warning(f"验证异常: {resp.status_code} {html}")

                if debug:
                    logger.debug(f'触发验证码: {html}')
                
                datadome = resp.cookies.get('datadome')

                nocaptcha_resp = (await _session.post(
                    f'http://{nocaptcha_host}/api/wanda/datadome/universal', headers={
                        'user-token': user_token,
                        **({ "Developer-Id": developer_id } if developer_id else {})
                    }, json={
                        'is_auth': auth,
                        "branch": branch,
                        "href": href,
                        "captcha_html": html,
                        "cookies": {
                            'datadome': datadome
                        },
                        "html": html,
                        "user_agent": user_agent,
                        "proxy": proxy,
                        **session.ipinfo,
                    }
                )).json()
                if debug:
                    logger.debug(nocaptcha_resp)
                
                if nocaptcha_resp["status"]:
                    extra = nocaptcha_resp["extra"]
                    session.cookies.update(nocaptcha_resp["data"])
                    if parse_index:
                        resp = await session.get(href, headers=headers)
                        retry_times += 1
                    else:
                        break            
                else:
                    raise Warning(f'验证失败, id: {nocaptcha_resp["id"]}, err: {nocaptcha_resp["msg"]}')
                
            if retry_times == 0 and js_url and js_key:
                datadome = session.cookies.get('datadome')
                _cookies = None
                if datadome:
                    _cookies = {
                        "datadome": datadome
                    }
                
                nocaptcha_resp = (await _session.post(
                    f'http://{nocaptcha_host}/api/wanda/datadome/universal', headers={
                        'user-token': user_token,
                        **({ "Developer-Id": developer_id } if developer_id else {})
                    }, json={
                        'is_auth': auth,
                        "branch": branch,
                        "href": href,
                        "js_url": js_url,
                        "js_key": js_key,
                        "cookies": _cookies,
                        "user_agent": user_agent,
                        "proxy": proxy,
                        **session.ipinfo,
                    }
                )).json()
                if debug:
                    logger.debug(nocaptcha_resp)
                    
                if nocaptcha_resp["status"]:
                    extra.update(nocaptcha_resp['extra'])
                    session.cookies.update(nocaptcha_resp['data'])
                else:
                    raise Warning(f'验证失败, id: {nocaptcha_resp["id"]}, err: {nocaptcha_resp["msg"]}')
        
        if parse_index:
            if interstitial:
                parse_index(extra)
            else:
                await parse_index(resp, extra)
        
        for verifier in verifiers:
            
            retry_times = 0
            
            resp = await verifier(session, extra)

            while resp.status_code in [403, 405] and retry_times < max_retry_times:
                did = extra.get("did")
                
                datadome = session.cookies["datadome"]
                
                captcha_args = {}
                try:
                    captcha_url = (await resp.async_json())["url"]
                    if 't=bv' in captcha_url:
                        raise Warning("代理被封锁, 请切换代理重试")
                    captcha_args["captcha_url"] = captcha_url
                except Warning as e:
                    raise e
                except:
                    captcha_html = await resp.async_text()
                    try:
                        dd_match = re.search(r'var dd=(\{.*?\})', captcha_html)
                        captcha = json.loads(dd_match[1].replace("'", '"'))
                        if captcha.get("t") == "bv":
                            raise Warning("代理被封锁, 请切换代理重试")
                    except:
                        raise Warning("验证异常: " + captcha_html)
                    
                    captcha_args["captcha_html"] = captcha_html
                
                if debug:
                    logger.debug(f'触发验证码: {captcha_args}')
                    
                nocaptcha_resp = (await _session.post(
                    f'http://{nocaptcha_host}/api/wanda/datadome/universal', headers={
                        'user-token': user_token,
                        **({ "Developer-Id": developer_id } if developer_id else {})
                    }, json={
                        'is_auth': auth,
                        "branch": branch,
                        "href": href,
                        **captcha_args,
                        "did": did,
                        "cookies": {
                            "datadome": datadome
                        },
                        "user_agent": user_agent,
                        "proxy": proxy,
                        **session.ipinfo,
                    }
                )).json()
                if debug:
                    logger.debug(nocaptcha_resp)
                    
                if nocaptcha_resp["status"]:
                    session.cookies.update(nocaptcha_resp['data'])
                    resp = await verifier(session, extra)
                else:
                    raise Warning(f'验证失败, id: {nocaptcha_resp["id"]}, err: {nocaptcha_resp["msg"]}')
                
                retry_times += 1

        return session, resp, extra



# -*- coding: utf-8 -*-


from http.cookies import SimpleCookie


class Cookie(dict):
    
    def to_str(self):
        """
        转成 headers 中的 cookie 字符串
        """
        cookie = SimpleCookie()
        for key, value in self.items():
            cookie[key] = value
        
        return cookie.output(header='', sep=';').strip().replace('="', '=').replace('"; ', '; ').strip('"')



# -*- coding: UTF-8 -*-

import sys
from curl_cffi import requests
from typing import Optional, Dict, Any, List
from pydantic import BaseModel
from loguru import logger

if sys.version_info >= (3, 8):
    from typing import Literal
else:
    from typing_extensions import Literal

import warnings
warnings.filterwarnings("ignore")


class KohlCheckBalanceParams(BaseModel):
    giftCardNumber: str
    giftCardPin: str
    g_recaptcha_response: str  # 使用下划线命名，便于代码提示
    
    def to_api_dict(self) -> dict:
        """转换为API需要的字典格式"""
        data = self.dict()
        # 处理特殊字段的命名
        if 'g_recaptcha_response' in data:
            data['g-recaptcha-response'] = data.pop('g_recaptcha_response')
        return data

class LululemonCheckBalanceParams(BaseModel):
    cardNumber: str
    pin: str

class WbiParams(BaseModel):
    cardNoH: str
    pinNoH: str
    g_recaptcha_response: str

    def to_api_dict(self) -> dict:
        """转换为API需要的字典格式"""
        data = self.dict()
        # 处理特殊字段的命名
        if 'g_recaptcha_response' in data:
            data['g-recaptcha-response'] = data.pop('g_recaptcha_response')
        return data

class DicksCheckBalanceParams(BaseModel):
    cardNumber: str
    pin: str

class FinishlineCheckBalanceParams(BaseModel):
    giftCardNumber: str
    giftCardPin: str

class FootlockerCheckBalanceParams(BaseModel):
    svcNumber: str
    svcPIN: str

class ReiCheckBalanceParams(BaseModel):
    giftCardNumber: str
    pin: str

class One4AllCheckBalanceParams(BaseModel):
    card_number: str
    card_cvv: str

class SaksCheckBalanceParams(BaseModel):
    token: str
    gcNumber: str
    gcPin: str

class CoachCheckBalanceParams(BaseModel):
    cardNumber: str
    pin: str

class FandangoCheckBalanceParams(BaseModel):
    code: str
    pin: str

class ColumbiaCheckBalanceParams(BaseModel):
    giftCertID: str
    giftCertPin: str

class HomeDepotCheckBalanceParams(BaseModel):
    card_number: str
    pin: str

class EtsyRegisterParams(BaseModel):
    first_name: str
    email: str
    password: str

class PrizepicksLoginParams(BaseModel):
    email: str
    password: str

class FifaLoginParams(BaseModel):
    email: str
    password: str

class AuspostTrackParams(BaseModel):
    tracking_ids: List[str]

class NorseSearchFlightsParams(BaseModel):
    currency: str
    departureDate: str
    returnDate: str
    destinations: str
    isOneWay: str
    origins: str
    residency: str
    adult: int
    partner: str

class ScootBookingParams(BaseModel):
    recordLocator: str
    firstName: str
    lastName: str
    origin: str
    destination: str
    hcaptchaToken: str
    isHcaptchaEnabled: bool

class MyPrepaidValidateParams(BaseModel):
    code: str

class PBandaiCartParams(BaseModel):
    cookies: Dict[str, Any]
    order: str
    model_no: str
    modelname: str

class MicrosoftCreateParams(BaseModel):
    BirthDate: str
    FirstName: str
    LastName: str
    MemberName: str
    Password: str

class GodaddyLoginParams(BaseModel):
    password: str
    username: str

class UltimateDiningLoginParams(BaseModel):
    email: str
    password: str

class FlightInfo(BaseModel):
    departurePort: str
    departureDate: str
    arrivalPort: str

class PassengerInfo(BaseModel):
    passengerType: str
    passengerSubType: Optional[str] = None
    quantity: int

class AjetAvailabilityParams(BaseModel):
    flights: List[FlightInfo]
    passengers: List[PassengerInfo]
    tripType: str

class AjetPnrLoginParams(BaseModel):
    bookingId: str
    surname: str
    recaptchaToken: str

class AjetGetBookingDetailParams(BaseModel):
    bookingId: str
    surname: str
    recaptchaToken: str

class AttCheckParams(BaseModel):
    userId: str
    domain: str

class EasyjetFindParams(BaseModel):
    bookingReference: str
    passengerLastName: str

class EurowingsBookingParams(BaseModel):
    identificationType: str
    locale: str
    processId: str
    bookingCode: str
    passengerLastName: str

class FastPeopleSearchParams(BaseModel):
    name: str
    address: str

class HiltonTokenParams(BaseModel):
    app_id: str

class HiltonCustomerParams(BaseModel):
    access_token: str

class KoreanAirLoginParams(BaseModel):
    userId: str
    password: str

class MaerskTokenParams(BaseModel):
    x_acm_username: str
    x_acm_password: str

    def to_api_dict(self) -> dict:
        """转换为API需要的字典格式"""
        data = self.dict()
        # 处理特殊字段的命名
        if 'x_acm_username' in data:
            data['x-acm-username'] = data.pop('x_acm_username')
        if 'x_acm_password' in data:
            data['x-acm-password'] = data.pop('x_acm_password')
        return data

class MarriottLoginParams(BaseModel):
    email: str
    password: str

class MarriottEnrollParams(BaseModel):
    email: str

class PokemonLoginParams(BaseModel):
    email: str
    password: str
    rechapchToken: str

class PokemonLotteryListParams(BaseModel):
    cookies: Dict[str, str]

class PokemonApplyLotteryParams(BaseModel):
    cookies: Dict[str, str]
    itemPrizeId: str
    jwt_token: str
    lotteryGroupId: str

class SephoraCheckBalanceParams(BaseModel):
    gcNumber: str
    gcPin: str
    captchaToken: str

class TextnowLoginParams(BaseModel):
    username: str
    password: str

class WalmartCheckBalanceParams(BaseModel):
    cardNumber: str
    pin: str

class WalmartCaCheckBalanceParams(BaseModel):
    cardNumber: str
    pin: str

class WalmartWalletParams(BaseModel):
    cookies: Dict[str, str]

class FlightSegment(BaseModel):
    arrivalStation: str
    departureDate: str
    departureStation: str

class WizzairSearchParams(BaseModel):
    adultCount: int
    childCount: int
    flightList: List[FlightSegment]
    infantCount: int
    isFlightChange: bool
    wdc: bool

class HmCheckBalanceParams(BaseModel):
    cardPin: str
    cardNumber: str


def run_unlocker(
    user_token: str,
    site: str,
    action: str,
    params: dict,
    user_agent: Optional[str] = None,
    proxy: Optional[str] = None,
    mode: Literal["generate", "cache"] = "cache",
    timeout: int = 60,
    branch: Optional[str] = None,
    auth: bool = False, 
    internal_host: bool = True,
    developer_id: Optional[str] = None,
    debug: bool = False
) -> dict:
    for k, v in list(params.items()):
        if callable(v):
            params[k] = v()
    
    headers = {
        'user-token': user_token
    }
    if developer_id:
        headers["Developer-Id"] = developer_id
    
    json_data = {
        'href': "https://" + site + "/",
        'action': action,
        'params': params,
        'user_agent': user_agent,
        'proxy': proxy
    }
    if mode == "generate":
        json_data["mode"] = "generate"
    
    if auth is True:
        json_data["is_auth"] = True
    
    if branch:
        json_data["branch"] = branch
    
    if internal_host:
        api = "http://api.nocaptcha.cn/api/wanda/site/unlocker"
    else:
        api = "http://api.nocaptcha.io/api/wanda/site/unlocker"
    
    resp = requests.post(api, headers=headers, json=json_data, timeout=timeout).json()
    if debug:
        logger.debug(resp)
        
    return resp


async def async_run_unlocker(
    user_token: str,
    site: str,
    action: str,
    params: dict,
    user_agent: Optional[str] = None,
    proxy: Optional[str] = None,
    mode: Literal["generate", "cache"] = "cache",
    timeout: int = 60,
    branch: Optional[str] = None,
    auth: bool = False, 
    internal_host: bool = True,
    developer_id: Optional[str] = None,
    debug: bool = False
) -> dict:
    for k, v in list(params.items()):
        if callable(v):
            params[k] = v()
    
    headers = {
        'user-token': user_token
    }
    if developer_id:
        headers["Developer-Id"] = developer_id
    
    json_data = {
        'href': "https://" + site + "/",
        'action': action,
        'params': params,
        'user_agent': user_agent,
        'proxy': proxy
    }
    if mode == "generate":
        json_data["mode"] = "generate"
    
    if auth is True:
        json_data["is_auth"] = True
    
    if branch:
        json_data["branch"] = branch
    
    if internal_host:
        api = "http://api.nocaptcha.cn/api/wanda/site/unlocker"
    else:
        api = "http://api.nocaptcha.io/api/wanda/site/unlocker"
    
    async with requests.AsyncSession() as session:
        resp = (await session.post(api, headers=headers, json=json_data, timeout=timeout)).json()
        
    if debug:
        logger.debug(resp)
    
    return resp


class Unlocker:
    
    def __init__(
        self, 
        user_token: str, 
        auth: bool = False,
        branch: Optional[str] = None,
        internal_host: bool = True,
        developer_id: Optional[str] = None,
        timeout: int = 60,
        debug: bool = False
    ):
        """
        @param user_token: 账号 token
        @param auth: 是否包月
        @param branch: 分支名
        @param internal_host: 是否国内域名
        @param developer_id: 开发者 id
        @param timeout: 超时时间: 秒
        @param debug: 是否输出结果
        """
        self._user_token = user_token
        self._auth = auth
        self._branch = branch
        self._internal_host = internal_host
        self._developer_id = developer_id
        self._timeout = timeout
        self._debug = debug
    
    def kohls_checkbalance(
        self, 
        params: KohlCheckBalanceParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Kohl's 查卡"""
        return run_unlocker(
            self._user_token, 'www.kohls.com', 'checkbalance', params.to_api_dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
    
    def lululemon_checkbalance(
        self, 
        params: LululemonCheckBalanceParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Lululemon 查卡"""
        return run_unlocker(
            self._user_token, 'shop.lululemon.com', 'checkbalance', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )

 
    def wbi_servlet(
        self, 
        params: WbiParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Adidas 查卡"""
        return run_unlocker(
            self._user_token, 'wbiprod.storedvalue.com', 'wb_servlet', params.to_api_dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )

 
    def dicks_checkbalance(
        self, 
        params: DicksCheckBalanceParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Dick's Sporting Goods 查卡"""
        return run_unlocker(
            self._user_token, 'www.dickssportinggoods.com', 'checkbalance', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )

 
    def finishline_checkbalance(
        self, 
        params: FinishlineCheckBalanceParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Finish Line 查卡"""
        return run_unlocker(
            self._user_token, 'www.finishline.com', 'checkbalance', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )

 
    def footlocker_checkbalance(
        self, 
        params: FootlockerCheckBalanceParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Foot Locker 查卡"""
        return run_unlocker(
            self._user_token, 'www.footlocker.com', 'checkbalance', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )

 
    def rei_checkbalance(
        self, 
        params: ReiCheckBalanceParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """REI 查卡"""
        return run_unlocker(
            self._user_token, 'www.rei.com', 'checkbalance', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )

 
    def one4all_checkbalance(
        self, 
        params: One4AllCheckBalanceParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """One4All 查卡"""
        return run_unlocker(
            self._user_token, 'www.one4all.com', 'checkbalance', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )

 
    def saks_checkbalance(
        self, 
        params: SaksCheckBalanceParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Saks Fifth Avenue 查卡"""
        return run_unlocker(
            self._user_token, 'www.saksfifthavenue.com', 'checkbalance', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )

 
    def coach_checkbalance(
        self, 
        params: CoachCheckBalanceParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Coach 查卡"""
        return run_unlocker(
            self._user_token, 'www.coach.com', 'checkbalance', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )

 
    def fandango_checkbalance(
        self, 
        params: FandangoCheckBalanceParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Fandango 查卡"""
        return run_unlocker(
            self._user_token, 'tickets.fandango.com', 'checkbalance', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )

 
    def columbia_checkbalance(
        self, 
        params: ColumbiaCheckBalanceParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Columbia 查卡"""
        return run_unlocker(
            self._user_token, 'www.columbia.com', 'checkbalance', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )

 
    def homedepot_checkbalance(
        self, 
        params: HomeDepotCheckBalanceParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Home Depot 查卡"""
        return run_unlocker(
            self._user_token, 'www.homedepot.com', 'recipient_experience_checkbalance', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )

 
    def etsy_register(
        self, 
        params: EtsyRegisterParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Etsy 注册"""
        return run_unlocker(
            self._user_token, 'www.etsy.com', 'register', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )

 
    def prizepicks_login(
        self, 
        params: PrizepicksLoginParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """PrizePicks 登录"""
        return run_unlocker(
            self._user_token, 'app.prizepicks.com', 'login', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )

 
    def fifa_login(
        self, 
        params: FifaLoginParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """FIFA 登录"""
        return run_unlocker(
            self._user_token, 'auth.fifa.com', 'login', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )

 
    def auspost_track(
        self, 
        params: AuspostTrackParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Auspost 追踪"""
        return run_unlocker(
            self._user_token, 'auspost.com.au', 'track', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )

 
    def norse_search_flights(
        self, 
        params: NorseSearchFlightsParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Norse 搜索航班"""
        return run_unlocker(
            self._user_token, 'connections.flynorse.com', 'search_flights', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )

 
    def scoot_booking_retrieval(
        self, 
        params: ScootBookingParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Scoot 预订检索"""
        return run_unlocker(
            self._user_token, 'manage.flyscoot.com', 'booking_retrieval', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )

 
    def myprepaid_validate(
        self, 
        params: MyPrepaidValidateParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """MyPrepaid 验证"""
        return run_unlocker(
            self._user_token, 'myprepaidcenter.com', 'validate_virtual_code', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )

 
    def pbandai_cart_add(
        self, 
        params: PBandaiCartParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """P-Bandai 购物车添加"""
        return run_unlocker(
            self._user_token, 'p-bandai.jp', 'pb_cart_add', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )

 
    def microsoft_create_account(
        self, 
        params: MicrosoftCreateParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Microsoft 创建账户"""
        return run_unlocker(
            self._user_token, 'signup.live.com', 'create_account', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )

 
    def godaddy_login(
        self, 
        params: GodaddyLoginParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """GoDaddy 登录"""
        return run_unlocker(
            self._user_token, 'sso.godaddy.com', 'login', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )

 
    def ultimatedining_login(
        self, 
        params: UltimateDiningLoginParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Ultimate Dining 登录"""
        return run_unlocker(
            self._user_token, 'theultimatediningcard.ca', 'login', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
 
    def ajet_getavailability(
        self, 
        params: AjetAvailabilityParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Ajet 查机票"""
        return run_unlocker(
            self._user_token, 'www.ajet.com', 'getavailability', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )

    def ajet_pnr_login(
        self, 
        params: AjetPnrLoginParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Ajet 查机票"""
        return run_unlocker(
            self._user_token, 'www.ajet.com', 'pnr_login', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )

    def ajet_get_booking_detail(
        self, 
        params: AjetGetBookingDetailParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Ajet 查机票"""
        return run_unlocker(
            self._user_token, 'www.ajet.com', 'get_booking_detail', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
        
    def att_check_availability(
        self, 
        params: AttCheckParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """AT&T 查机票"""
        return run_unlocker(
            self._user_token, 'www.att.com', 'check_availability', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )

    def easyjet_find_booking(
        self, 
        params: EasyjetFindParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """EasyJet 查找预订"""
        return run_unlocker(
            self._user_token, 'www.easyjet.com', 'find_booking', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
        
    def eurowings_openbooking(
        self, 
        params: EurowingsBookingParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Eurowings 打开预订"""
        return run_unlocker(
            self._user_token, 'www.eurowings.com', 'openbooking', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )

    def fastpeoplesearch_search(
        self, 
        params: FastPeopleSearchParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """FastPeopleSearch 搜索"""
        return run_unlocker(
            self._user_token, 'www.fastpeoplesearch.com', 'search_people', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )

    def hilton_get_token(
        self, 
        params: HiltonTokenParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Hilton 获取令牌"""
        return run_unlocker(
            self._user_token, 'www.hilton.com', 'get_web_guest_token', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )

    def hilton_new_customer(
        self, 
        params: HiltonCustomerParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Hilton 新客户"""
        return run_unlocker(
            self._user_token, 'www.hilton.com', 'new_customer', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )

    def koreanair_login(
        self, 
        params: KoreanAirLoginParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Korean Air 登录"""
        return run_unlocker(
            self._user_token, 'www.koreanair.com.cn', 'cn_login', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )

    def maersk_get_token(
        self, 
        params: MaerskTokenParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Maersk 获取令牌"""
        return run_unlocker(
            self._user_token, 'www.maersk.com', 'get_access_token', params.to_api_dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )

    def marriott_login(
        self, 
        params: MarriottLoginParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Marriott 登录"""
        return run_unlocker(
            self._user_token, 'www.marriott.com', 'login', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )

    def marriott_enroll(
        self, 
        params: MarriottEnrollParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Marriott 注册"""
        return run_unlocker(
            self._user_token, 'www.marriott.com', 'enroll', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )

    def pokemon_login(
        self, 
        params: PokemonLoginParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Pokemon Center 登录"""
        return run_unlocker(
            self._user_token, 'www.pokemoncenter-online.com', 'login', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )

    def pokemon_get_lottery_list(
        self, 
        params: PokemonLotteryListParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Pokemon Center 获取抽奖列表"""
        return run_unlocker(
            self._user_token, 'www.pokemoncenter-online.com', 'get_lottery_list', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )

    def pokemon_apply_lottery(
        self, 
        params: PokemonApplyLotteryParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Pokemon Center 申请抽奖"""
        return run_unlocker(
            self._user_token, 'www.pokemoncenter-online.com', 'apply_lottery', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )

    def sephora_checkbalance(
        self, 
        params: SephoraCheckBalanceParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Sephora 查卡"""
        return run_unlocker(
            self._user_token, 'www.sephora.com', 'checkbalance', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )

    def textnow_login(
        self, 
        params: TextnowLoginParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """TextNow 登录"""
        return run_unlocker(
            self._user_token, 'www.textnow.com', 'login', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )

    def walmart_checkbalance(
        self, 
        params: WalmartCheckBalanceParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Walmart 查卡"""
        return run_unlocker(
            self._user_token, 'www.walmart.com', 'checkbalance', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )

    def walmart_ca_checkbalance(
        self, 
        params: WalmartCaCheckBalanceParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Walmart Canada 查卡"""
        return run_unlocker(
            self._user_token, 'www.walmart.ca', 'ca_checkbalance', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )

    def walmart_get_wallet(
        self, 
        params: WalmartWalletParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Walmart 获取钱包"""
        return run_unlocker(
            self._user_token, 'www.walmart.com', 'get_wallet_payments', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )

    def wizzair_search_flights(
        self, 
        params: WizzairSearchParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Wizzair 搜索航班"""
        return run_unlocker(
            self._user_token, 'www.wizzair.com', 'search_flights', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )

    def hm_checkbalance(
        self, 
        params: HmCheckBalanceParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """H&M 查卡"""
        return run_unlocker(
            self._user_token, 'www2.hm.com', 'checkbalance', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
        

class AsyncUnlocker:
    
    def __init__(
        self, 
        user_token: str, 
        auth: bool = False,
        timeout: int = 60,
        branch: Optional[str] = None,
        internal_host: bool = True,
        developer_id: Optional[str] = None,
        debug: bool = False
    ):
        """
        @param user_token: 账号 token
        @param auth: 是否包月
        @param branch: 分支名
        @param internal_host: 是否国内域名
        @param developer_id: 开发者 id
        @param timeout: 超时时间: 秒
        @param debug: 是否输出结果
        """
        self._user_token = user_token
        self._auth = auth
        self._branch = branch
        self._internal_host = internal_host
        self._developer_id = developer_id
        self._timout = timeout
        self._debug = debug
        
    async def kohls_checkbalance(
        self, 
        params: KohlCheckBalanceParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Kohl's 查卡"""
        return await async_run_unlocker(
            self._user_token, 'www.kohls.com', 'checkbalance', params.to_api_dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )

    async def lululemon_checkbalance(
        self, 
        params: LululemonCheckBalanceParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Lululemon 查卡"""
        return await async_run_unlocker(
            self._user_token, 'shop.lululemon.com', 'checkbalance', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
    
    async def wbi_servlet(
        self, 
        params: WbiParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Adidas 查卡"""
        return await async_run_unlocker(
            self._user_token, 'wbiprod.storedvalue.com', 'wb_servlet', params.to_api_dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
    
    async def dicks_checkbalance(
        self, 
        params: DicksCheckBalanceParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Dick's Sporting Goods 查卡"""
        return await async_run_unlocker(
            self._user_token, 'www.dickssportinggoods.com', 'checkbalance', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
    
    async def finishline_checkbalance(
        self, 
        params: FinishlineCheckBalanceParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Finish Line 查卡"""
        return await async_run_unlocker(
            self._user_token, 'www.finishline.com', 'checkbalance', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
    
    async def footlocker_checkbalance(
        self, 
        params: FootlockerCheckBalanceParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Foot Locker 查卡"""
        return await async_run_unlocker(
            self._user_token, 'www.footlocker.com', 'checkbalance', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
    
    async def rei_checkbalance(
        self, 
        params: ReiCheckBalanceParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """REI 查卡"""
        return await async_run_unlocker(
            self._user_token, 'www.rei.com', 'checkbalance', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
    
    async def one4all_checkbalance(
        self, 
        params: One4AllCheckBalanceParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """One4All 查卡"""
        return await async_run_unlocker(
            self._user_token, 'www.one4all.com', 'checkbalance', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
    
    async def saks_checkbalance(
        self, 
        params: SaksCheckBalanceParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Saks Fifth Avenue 查卡"""
        return await async_run_unlocker(
            self._user_token, 'www.saksfifthavenue.com', 'checkbalance', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
    
    async def coach_checkbalance(
        self, 
        params: CoachCheckBalanceParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Coach 查卡"""
        return await async_run_unlocker(
            self._user_token, 'www.coach.com', 'checkbalance', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
    
    async def fandango_checkbalance(
        self, 
        params: FandangoCheckBalanceParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Fandango 查卡"""
        return await async_run_unlocker(
            self._user_token, 'tickets.fandango.com', 'checkbalance', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
    
    async def columbia_checkbalance(
        self, 
        params: ColumbiaCheckBalanceParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Columbia 查卡"""
        return await async_run_unlocker(
            self._user_token, 'www.columbia.com', 'checkbalance', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
    
    async def homedepot_checkbalance(
        self, 
        params: HomeDepotCheckBalanceParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Home Depot 查卡"""
        return await async_run_unlocker(
            self._user_token, 'www.homedepot.com', 'recipient_experience_checkbalance', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
    
    async def etsy_register(
        self, 
        params: EtsyRegisterParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Etsy 注册"""
        return await async_run_unlocker(
            self._user_token, 'www.etsy.com', 'register', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
    
    async def prizepicks_login(
        self, 
        params: PrizepicksLoginParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """PrizePicks 登录"""
        return await async_run_unlocker(
            self._user_token, 'app.prizepicks.com', 'login', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
    
    async def fifa_login(
        self, 
        params: FifaLoginParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """FIFA 登录"""
        return await async_run_unlocker(
            self._user_token, 'auth.fifa.com', 'login', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
    
    async def auspost_track(
        self, 
        params: AuspostTrackParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Auspost 追踪"""
        return await async_run_unlocker(
            self._user_token, 'auspost.com.au', 'track', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
    
    async def norse_search_flights(
        self, 
        params: NorseSearchFlightsParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Norse 搜索航班"""
        return await async_run_unlocker(
            self._user_token, 'connections.flynorse.com', 'search_flights', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
    
    async def scoot_booking_retrieval(
        self, 
        params: ScootBookingParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Scoot 预订检索"""
        return await async_run_unlocker(
            self._user_token, 'manage.flyscoot.com', 'booking_retrieval', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
    
    async def myprepaid_validate(
        self, 
        params: MyPrepaidValidateParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """MyPrepaid 验证"""
        return await async_run_unlocker(
            self._user_token, 'myprepaidcenter.com', 'validate_virtual_code', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
    
    async def pbandai_cart_add(
        self, 
        params: PBandaiCartParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """P-Bandai 购物车添加"""
        return await async_run_unlocker(
            self._user_token, 'p-bandai.jp', 'pb_cart_add', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
    
    async def microsoft_create_account(
        self, 
        params: MicrosoftCreateParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Microsoft 创建账户"""
        return await async_run_unlocker(
            self._user_token, 'signup.live.com', 'create_account', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
    
    async def godaddy_login(
        self, 
        params: GodaddyLoginParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """GoDaddy 登录"""
        return await async_run_unlocker(
            self._user_token, 'sso.godaddy.com', 'login', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
    
    async def ultimatedining_login(
        self, 
        params: UltimateDiningLoginParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Ultimate Dining 登录"""
        return await async_run_unlocker(
            self._user_token, 'theultimatediningcard.ca', 'login', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
    
    async def ajet_getavailability(
        self, 
        params: AjetAvailabilityParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Ajet 查机票"""
        return await async_run_unlocker(
            self._user_token, 'www.ajet.com', 'getavailability', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
    
    async def att_check_availability(
        self, 
        params: AttCheckParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """AT&T 查机票"""
        return await async_run_unlocker(
            self._user_token, 'www.att.com', 'check_availability', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
    
    async def easyjet_find_booking(
        self, 
        params: EasyjetFindParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """EasyJet 查找预订"""
        return await async_run_unlocker(
            self._user_token, 'www.easyjet.com', 'find_booking', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
    
    async def eurowings_openbooking(
        self, 
        params: EurowingsBookingParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Eurowings 打开预订"""
        return await async_run_unlocker(
            self._user_token, 'www.eurowings.com', 'openbooking', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
    
    async def fastpeoplesearch_search(
        self, 
        params: FastPeopleSearchParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """FastPeopleSearch 搜索"""
        return await async_run_unlocker(
            self._user_token, 'www.fastpeoplesearch.com', 'search_people', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
    
    async def hilton_get_token(
        self, 
        params: HiltonTokenParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Hilton 获取令牌"""
        return await async_run_unlocker(
            self._user_token, 'www.hilton.com', 'get_web_guest_token', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
    
    async def hilton_new_customer(
        self, 
        params: HiltonCustomerParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Hilton 新客户"""
        return await async_run_unlocker(
            self._user_token, 'www.hilton.com', 'new_customer', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
    
    async def koreanair_login(
        self, 
        params: KoreanAirLoginParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Korean Air 登录"""
        return await async_run_unlocker(
            self._user_token, 'www.koreanair.com.cn', 'cn_login', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
    
    async def maersk_get_token(
        self, 
        params: MaerskTokenParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Maersk 获取令牌"""
        return await async_run_unlocker(
            self._user_token, 'www.maersk.com', 'get_access_token', params.to_api_dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
    
    async def marriott_login(
        self, 
        params: MarriottLoginParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Marriott 登录"""
        return await async_run_unlocker(
            self._user_token, 'www.marriott.com', 'login', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
    
    async def marriott_enroll(
        self, 
        params: MarriottEnrollParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Marriott 注册"""
        return await async_run_unlocker(
            self._user_token, 'www.marriott.com', 'enroll', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
    
    async def pokemon_login(
        self, 
        params: PokemonLoginParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Pokemon Center 登录"""
        return await async_run_unlocker(
            self._user_token, 'www.pokemoncenter-online.com', 'login', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
    
    async def pokemon_get_lottery_list(
        self, 
        params: PokemonLotteryListParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Pokemon Center 获取抽奖列表"""
        return await async_run_unlocker(
            self._user_token, 'www.pokemoncenter-online.com', 'get_lottery_list', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
    
    async def pokemon_apply_lottery(
        self, 
        params: PokemonApplyLotteryParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Pokemon Center 申请抽奖"""
        return await async_run_unlocker(
            self._user_token, 'www.pokemoncenter-online.com', 'apply_lottery', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
    
    async def sephora_checkbalance(
        self, 
        params: SephoraCheckBalanceParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Sephora 查卡"""
        return await async_run_unlocker(
            self._user_token, 'www.sephora.com', 'checkbalance', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
    
    async def textnow_login(
        self, 
        params: TextnowLoginParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """TextNow 登录"""
        return await async_run_unlocker(
            self._user_token, 'www.textnow.com', 'login', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
    
    async def walmart_checkbalance(
        self, 
        params: WalmartCheckBalanceParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Walmart 查卡"""
        return await async_run_unlocker(
            self._user_token, 'www.walmart.com', 'checkbalance', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
    
    async def walmart_ca_checkbalance(
        self, 
        params: WalmartCaCheckBalanceParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Walmart Canada 查卡"""
        return await async_run_unlocker(
            self._user_token, 'www.walmart.ca', 'ca_checkbalance', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
    
    async def walmart_get_wallet(
        self, 
        params: WalmartWalletParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Walmart 获取钱包"""
        return await async_run_unlocker(
            self._user_token, 'www.walmart.com', 'get_wallet_payments', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
    
    async def wizzair_search_flights(
        self, 
        params: WizzairSearchParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """Wizzair 搜索航班"""
        return await async_run_unlocker(
            self._user_token, 'www.wizzair.com', 'search_flights', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
    
    async def hm_checkbalance(
        self, 
        params: HmCheckBalanceParams, 
        user_agent: Optional[str] = None,
        proxy: Optional[str] = None, 
        mode: Literal["generate", "cache"] = "cache"
    ) -> dict:
        """H&M 查卡"""
        return await async_run_unlocker(
            self._user_token, 'www2.hm.com', 'checkbalance', params.dict(), 
            user_agent=user_agent, proxy=proxy, auth=self._auth, mode=mode, timeout=self._timeout, branch=self._branch, internal_host=self._internal_host, 
            developer_id=self._developer_id, debug=self._debug
        )
